#!/usr/bin/env python3

import os

#os.system("sudo update-alternatives --install /usr/bin/python python /usr/bin/python3 1")

deplist =  ["boost"]
deplist += ["wget"]
deplist += ["texinfo"]
deplist += ["m4","bison","flex"]
deplist += ["xorg-x11"]
deplist += ["xorg-server"]
deplist += ["xorg-proto"]
deplist += ["xcb-proto"]
deplist += ["dev-libs/openssl"]
deplist += ["dev-cpp/tbb"]

merged = " ".join(deplist)
os.system("emerge %s" % merged)



#!/usr/bin/env python3 

from obt import path, pathtools, macos, dep
import os, argparse

parser = argparse.ArgumentParser(description='obt.d build')
parser.add_argument('--alllibs', action="store_true", help='do all libs' )
parser.add_argument('--obt.ibs', action="store_true", help='do obt.d libs' )
parser.add_argument('--obt.ymods', action="store_true", help='do obt.d python modules' )

_args = vars(parser.parse_args())

if _args["alllibs"]!=False:
  for item in pathtools.patglob(path.stage()/"lib","*.dylib"):
    macos.macho_replace_loadpaths(item,"@executable_path/../lib","@rpath")
    macos.macho_dump(item)

if _args["obt.ibs"]!=False:
  for item in pathtools.patglob(path.stage()/"lib","libobt..dylib"):
    macos.macho_replace_loadpaths(item,"@executable_path/../lib","@rpath")
    macos.macho_dump(item)

if _args["obt.ymods"]!=False:
  PYTHON = dep.instance("python")

  for item in pathtools.patglob(PYTHON.site_packages_dir/"obt.ngine"/"core","*.so"):
    macos.macho_replace_loadpaths(item,"@executable_path/../lib","@rpath")
    macos.macho_dump(item)

  for item in pathtools.patglob(PYTHON.site_packages_dir/"obt.ngine"/"lev2","*.so"):
    macos.macho_replace_loadpaths(item,"@executable_path/../lib","@rpath")
    macos.macho_dump(item)




from obt import path, pathtools, command, dep, host
from yarl import URL

class ngc(dep.Provider):
  def __init__(self): ############################################
    super().__init__("ngc")
    self.build_dest = path.builds()/"ngc"
    self.manifest = path.manifests()/"ngc"
    self.OK = self.manifest.exists()
    self.dest_bin = path.obt_bin()/"ngc"
  def __str__(self):
    return "NGC (nvidia-wget)"
  def download_and_extract(self): #############################################
    dlbase = URL("https://ngc.nvidia.com/downloads")
    HASH = "0"
    fname = ""
    if host.IsOsx and host.IsX86_64:
      HASH = "899ebcbb4743620d52ef1f7e9574b070"
      fname = "ngccli_mac.zip"
    elif host.IsLinux and host.IsX86_64:
      HASH = "17b45d020650adbab3dda22eb271bc39"
      fname = "ngccli_linux.zip"

    url = dlbase/fname
    self.arcpath = dep.downloadAndExtract([url],
                                          fname,
                                          "zip",
                                          HASH,
                                          self.build_dest)
  def build(self): ############################################################
    command.run(["unzip","-o",self.arcpath],working_dir=self.build_dest)
    pathtools.copyfile(self.build_dest/"ngc",self.dest_bin,"u+x")
    "ngc config set"
    "distribution=$(. /etc/os-release;echo $ID$VERSION_ID) \
      && curl -fsSL https://nvidia.github.io/libnvidia-container/gpgkey | sudo gpg --dearmor -o /usr/share/keyrings/nvidia-container-toolkit-keyring.gpg \
      && curl -s -L https://nvidia.github.io/libnvidia-container/$distribution/libnvidia-container.list | \
            sed 's#deb https://#deb [signed-by=/usr/share/keyrings/nvidia-container-toolkit-keyring.gpg] https://#g' | \
            sudo tee /etc/apt/sources.list.d/nvidia-container-toolkit.list"

    return self.dest_bin.exists()
  def provide(self): ##########################################################
    if False==self.OK:
      self.download_and_extract()
      self.OK = self.build()
      if self.OK:
        self.manifest.touch()
    return self.OK



cd ${OBT_BUILDS}
git clone git://github.com/FFmpeg/FFmpeg.git
cd FFmpeg/
./configure --prefix=${OBT_STAGE} --enable-nonfree --enable-shared --enable-debug=1 --disable-stripping --assert-level=1 --enable-memory-poisoning --disable-optimizations
make -j 16
make install



#!/usr/bin/env python3

###############################################################################
# Orkid Build System
# Copyright 2010-2022, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import obt.host, obt.sdk

from obt import dep, path
from obt.deco import Deco
deco = Deco()


h = obt.host.description()
host_sdk = obt.sdk.descriptor(h.architecture,h.os)

host_id = "%s.%s" % (h.architecture,h.os)

def print_item(key,val):
 dstr = deco.inf(host_id)
 kstr = deco.key(key)
 vstr = deco.val(val)
 print("%s.%s = %s"%(dstr,kstr,vstr))

print_item("architecture",h.architecture)
print_item("os",h.os)
print_item("revision",h.revision)
print_item("codename",h.codename)

targets = list()

for item in h.targets:
	tname = item.identifier
	targets += [tname]

print_item("targets",targets)

print_item("hostsdk.c_compiler",host_sdk.c_compiler)
print_item("hostsdk.cxx_compiler",host_sdk.cxx_compiler)




###############################################################################
# Orkid Build System
# Copyright 2010-2022, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os, multiprocessing 
from obt import target, osrelease

class hostinfo:
  def __init__(self):
    self.architecture = "x86_64"
    self.os = "linux"
    descriptor = osrelease.descriptor()
    self.distribution = descriptor.id
    self.revision = descriptor.version_id
    self.codename = descriptor.version_codename
    self.id = "x86_64"
    ###################################
    x86_64_linux = target.descriptor('x86_64','linux')
    aarch64_android = target.descriptor('aarch64','android')
    targets = set()
    targets.add(x86_64_linux)
    targets.add(aarch64_android)
    self.targets = targets
    self.target = x86_64_linux # host SDK
    ###################################

  @property
  def is_gentoo(self):
    portage_exists = os.path.exists("/etc/portage")
    return portage_exists
  @property
  def is_debian(self):
    apt_exists = os.path.exists("/etc/apt")
    return apt_exists
  @property
  def num_cores(self):
    return multiprocessing.cpu_count()
  @property
  def identifier(self):
    return "x86_64-linux"



###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import os, inspect, tarfile
from pathlib import Path
import importlib.util
import obt.path, obt.host
from obt.command import Command, run
from obt.deco import Deco
from obt.wget import wget
from obt import pathtools, cmake, make, path, git, host, buildtrace
import obt._dep_enumerate

deco = Deco()

###############################################################################

def switch(linux=None,macos=None):
  if host.IsOsx:
    if macos==None:
      assert(False) # dep not supported on platform
    return macos
  elif host.IsIx:
    if linux==None:
      assert(False) # dep not supported on platform
    return linux
  else:
   return linux


###############################################################################
# require - returns True if dep(s) succesfully provided, otherwise False
###############################################################################

def require(name_or_list):
  from obt import _dep_node
  ######################################
  # list of dependencies?
  ######################################
  with buildtrace.NestedBuildTrace({ "op": "dep.require(%s)"%str(name_or_list)}) as nested:
   if (isinstance(name_or_list,list)):
    ######################
    # empty list ? 
    ######################
    if not name_or_list:
      return True # no deps, requirements met..
    ######################
    for item in name_or_list:
      #print(item)

      inst = _dep_node.DepNode.FIND(item)
      if inst==None:
        deconame = deco.orange(item)
        print(deco.red("dep not found>> %s"%deconame))
        return False
      rval = inst.provide()
      ######################
      # a dep failed....
      ######################
      if not rval:
        return False
    ######################
    # all deps must be met!
    ######################
    return True
   ######################################
   # single dependency (string) ?
   ######################################
   else:
    inst = _dep_node.DepNode.FIND(name_or_list)
    #######################
    ## dep not found...
    #######################
    if inst==None:
      deconame = deco.orange(name_or_list)
      print(deco.red("dep not found>> %s"%deconame))
      return False
    #######################
    ## dep was found, is it provided ?
    #######################
    OK = inst.provide()
    return inst.instance if OK else None
   #######################
   # technically, this should never hit..
   #######################
  assert(False)
  return False




#!/usr/bin/env python3

import os, sys, pathlib, argparse
from obt import subspace, host, path
import obt.pathtools
import obt._globals

##########################################
# build dep dict
##########################################

subspacemodules = subspace.enumerate()

##########################################

import obt.deco
deco = obt.deco.Deco()

#print(subspacemodules)

line_index = 0
for key in subspacemodules:
  odd = line_index&1
  val = str(subspacemodules[key]._module.info())
  if val!="???":
    col_k = deco.rgbstr(255,255,0,key) if odd else deco.rgbstr(192,192,0,key)
    col_v = deco.rgbstr(255,255,255,val) if odd else deco.rgbstr(192,192,192,val)
    line = "%27s" % col_k
    line += " " + deco.magenta(":") + " "
    line += col_v
    print(line)
    line_index += 1



###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################
from obt import dep, command, pathtools, path, host
###############################################################################
class rapidjson(dep.StdProvider):
  VERSION ="master"
  NAME = "rapidjson"
  def __init__(self):
    super().__init__(rapidjson.NAME)
    self.declareDep("cmake")
    self._builder = dep.CMakeBuilder(rapidjson.NAME)
    self._builder.setCmVars({
        "CMAKE_BUILD_TYPE": "RELEASE",
    })
    if host.IsDarwin:
      osxname = host.description().codename
      self._builder.setCmVars({
        "RAPIDJSON_BUILD_THIRDPARTY_GTEST": "OFF",
        "RAPIDJSON_BUILD_TESTS": "OFF",
      })
      if osxname=="monterey" or osxname=="ventura":
        self._builder.setCmVars({
          "CMAKE_CXX_FLAGS": "-Wno-deprecated-declarations -Wno-deprecated-copy-with-user-provided-copy"
        })
    else:
      self._builder.setCmVars({
        "CMAKE_CXX_FLAGS": "-Wno-stringop-overflow -Wno-array-bounds",
        "BUILD_EXAMPLES": "ON",
      })
  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=rapidjson.NAME,
                             repospec="tweakoz/rapidjson",
                             revision=rapidjson.VERSION,
                             recursive=True)

  ########################################################################
  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"CMakeLists.txt").exists()
  def areRequiredBinaryFilesPresent(self):
    return (path.includes()/"rapidjson"/"rapidjson.h").exists()



###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################
from obt import dep, log, path
###############################################################################
class cycles(dep.StdProvider):
  name = "cycles"
  def __init__(self):
    super().__init__(cycles.name)
    self.declareDep("boost")
    self.declareDep("pugixml")
    self.declareDep("openjpeg")
    self.declareDep("osl")
    self.declareDep("ocio")
    self._builder = self.createBuilder(dep.CMakeBuilder)
    self._builder.setCmVar("CMAKE_MODULE_PATH",path.stage()/"lib"/"cmake")
    self._builder.setCmVar("BOOST_ROOT",path.stage())
    self._builder.setCmVar("Boost_INCLUDE_DIR",path.includes()/"boost")
    self._builder.setCmVar("Boost_NO_SYSTEM_PATHS","ON")
    #self._builder.setCmVar("Boost_DEBUG","1")

  ########################################################################
  @property
  def _fetcher(self):
    fetcher = dep.GitFetcher(cycles.name)
    fetcher._git_url = "https://git.blender.org/cycles.git"
    fetcher._cache=False,
    fetcher._recursive=False
    fetcher._revision = "master"
    return fetcher
  ########################################################################

  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"CMakeLists.txt").exists()

  def areRequiredBinaryFilesPresent(self):
    return (path.stage()/"bin"/"llvm-cov").exists()
###############################################################################



###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################
from obt import dep, log, path, template
###############################################################################
class opensubdiv(dep.StdProvider):
  name = "opensubdiv"
  def __init__(self):
    super().__init__(opensubdiv.name)
    #self.declareDep("llvm")
    self.declareDep("cmake")
    cuda = self.declareDep("cuda")
    self._builder = self.createBuilder(dep.CMakeBuilder)
    self._builder._cmakeenv = {
      "CMAKE_CXX_COMPILER": cuda.cxx_compiler,
      "CMAKE_C_COMPILER": cuda.c_compiler,
      "CMAKE_CXX_STANDARD": "17",
      "PXR_ENABLE_PYTHON_SUPPORT": "OFF",
      "BOOST_ROOT": path.stage(),
      "Boost_NO_SYSTEM_PATHS": "ON",
      "OSD_CUDA_NVCC_FLAGS": "--gpu-architecture compute_30"
      #"Boost_DEBUG":"ON"
    }

  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=opensubdiv.name,
                             repospec="PixarAnimationStudios/OpenSubdiv",
                             revision="release",
                             recursive=True)

  ########################################################################
  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"CMakeLists.txt").exists()

  def areRequiredBinaryFilesPresent(self):
    return (path.libs()/"libosdGPU.so").exists()
###############################################################################





#!/usr/bin/env python3 

import os
import sys
import site
import obt.path

def print_env_var(name, default):
    value = os.getenv(name, default)
    print(f"{name}: {value}")

print( "######################################################")

print_env_var('PYTHONPATH', sys.path)
print_env_var('PYTHONHOME', sys.prefix)
print_env_var('PYTHONSTARTUP', 'Not set')
print_env_var('PYTHONUSERBASE', site.USER_BASE)
print_env_var('PYTHONEXECUTABLE', sys.executable)
print_env_var('PYTHONWARNINGS', 'Not set')
print_env_var('PYTHONNOUSERSITE', 'Not set (User site directory is added to sys.path)')
print_env_var('PYTHONUNBUFFERED', 'Not set (Buffered I/O is used for stdout and stderr)')

print_env_var('site.PREFIXES', site.PREFIXES)
print_env_var('site.USER_SITE', site.USER_SITE)
print_env_var('site.USER_BASE', site.USER_BASE)
print_env_var('sys.prefix', sys.prefix)
print_env_var('sys.base_prefix', sys.base_prefix)

print( "######################################################")

print( "obt-pymodule-path: %s" % obt.path.obt_module_path() )
print( "obt-data-base: %s" % obt.path.obt_data_base() )
print( "obt-modules-base: %s" % obt.path.obt_modules_base() )
print( "running_from_pip: %s" % obt.path.running_from_pip() )


from obt import dep, pip, path, dep, host
from obt.command import Command
###############################################################################

class pydefaults(dep.Provider):

  def __init__(self): ############################################
    super().__init__("pydefaults")
    build_dest = path.builds()/"pydefaults"
    self.build_dest = build_dest
    self.manifest = path.manifests()/"pydefaults"
    self.OK = self.manifest.exists()
    self.python = self.declareDep("python")

  def build(self): ############################################################
      #PYTHON = dep.instance("python")
      pip.install(["pytest",
                   "numpy","scipy",
                   "numba","pyopencl",
                   "matplotlib",
                   "pyzmq"])#,"backports.lzma"])



      #################
      modules2 = ["Pillow","jupyter","plotly","trimesh","asciidoc", "pyudev"]
      if host.IsDarwin == False:
        modules2 += ["pysqlite3"]
      #################

      ret = Command([self.python.executable,"-m","pip","install","--upgrade"]+modules2).exec()

      print("pydefaults build ret<%d>"%int(ret))
      return (ret==0)

  def areRequiredSourceFilesPresent(self):
    return (self.python.site_packages_dir/"numpy"/"LICENSE.txt").exists()

  def areRequiredBinaryFilesPresent(self):
    return self.areRequiredSourceFilesPresent()

###############################################################################



import os, subprocess

########################################################

def procargs(command_list):
  newlist = []
  for item in command_list:
      newlist.append(str(item))
  return newlist

########################################################

def sync_subprocess(command_list):
  cmd_list = " ".join(procargs(command_list))
  print(command_list)
  print(cmd_list)
  child_process = subprocess.Popen( cmd_list,
                                    universal_newlines=True,
                                    env=os.environ,
                                    shell=True )
  child_process.communicate()
  child_process.wait()


